%-------------------------------------------------------------------------
% FILE: slam.m
% AUTH: B. Green
% DATE: 07/28/06
% DESC: Implements the SLAM algorithm based on a robot from "A Solution 
%       to the Simultaneous Localization and Map Building Problem".
%-------------------------------------------------------------------------

clear all; clc;

N = 10;
deltaT=1;

%--------------------------PROCESS MODEL----------------------------------
x = zeros(1,N);
y = zeros(1,N);
phi = zeros(1,N);

% Constants
L = 2.83;       % distance [m] between front and rear axle
a = L - 0.95;   % distance [m] from rear axle to radar
b = 0.5;        % distance [m] from longitudinal axis of truck to laser

vel = 3 + zeros(1,N);       % velocity input (V=3)
gamma = [0.5, 0.5, 0, 0, -0.8, 0, 0.8, -0.3, -0.8, -0.8];   % steering input

for i=2:N
    x(i) = x(i-1) + deltaT*vel(i-1)*cos(phi(i-1));
    y(i) = y(i-1) + deltaT*vel(i-1)*sin(phi(i-1));
    phi(i) = phi(i-1) + (deltaT*vel(i-1)*tan(gamma(i-1)))/L;
end
%-------------------------------------------------------------------------

%-----------------SIMULATE NOISY RADAR MEASUREMENT------------------------
LM1_x = 4; LM1_y = 4;   % landmark 1
LM2_x = 12; LM2_y = 7;  % landmark 2
LM3_x = 13; LM3_y = 14; % landmark 3

for i=1:N
    r(1,i) = sqrt((LM1_x-x(i))^2 + (LM1_y-y(i))^2);
    r(2,i) = sqrt((LM2_x-x(i))^2 + (LM2_y-y(i))^2);
    r(3,i) = sqrt((LM3_x-x(i))^2 + (LM3_y-y(i))^2);
    
    if(i==1) heading = 0;
    else heading = atan2(y(i)-y(i-1), x(i)-x(i-1)); end
    
    % theta for landmark 1    
    theta(1,i) = atan2(LM1_y-y(i), LM1_x-x(i));
    if(theta(1,i)<0) theta(1,i)=theta(1,i)+360*pi/180; end
    if(theta(1,i)<heading)
        theta(1,i) = 360*pi/180 + (theta(1,i) - heading);
    else
        theta(1,i) = theta(1,i) - heading;
    end
    
    % theta for landmark 2    
    theta(2,i) = atan2(LM2_y-y(i), LM2_x-x(i));
    if(theta(2,i)<0) theta(2,i)=theta(2,i)+360*pi/180; end    
    if(theta(2,i)<heading)
        theta(2,i) = 360*pi/180 + (theta(2,i) - heading);
    else
        theta(2,i) = theta(2,i) - heading;
    end
    
    % theta for landmark 3    
    theta(3,i) = atan2(LM3_y-y(i), LM3_x-x(i));
    if(theta(3,i)<0) theta(3,i)=theta(3,i)+360*pi/180; end    
    if(theta(3,i)<heading)
        theta(3,i) = 360*pi/180 + (theta(3,i) - heading);
    else
        theta(3,i) = theta(3,i) - heading;
    end
    
end

r
theta

%r(1,:) = [5,4,3,3,5,7.1,10.5,13.2,16,19.2];        % distance to landmark 1
%theta(1,:) = [40,80,90,115,165,164,220,200,148,175];   % bearing to landmark 1
%r(2,:) = [13,11,8,4.95,4.5,3.7,3.2,4.4,7.9,11];       % distance to landmark 2
%theta(2,:) = [30,50,10,315,300,245,260,220,160,190];   % bearing to landmark 1
%r(3,:) = [20,19,13,11.7,9,5.7,4.2,4.1,3.7,5.6];       % distance to landmark 3
%theta(3,:) = [40,60,25,345,340,345,80,120,102,139];   % bearing to landmark 3
%-------------------------------------------------------------------------


% Plot results
t = 0:1:N-1;
figure; hold on;
plot(x,y,'-*');
plot(4,4,'o');      % landmark 1
plot(12,7,'o');     % landmark 2
plot(13,14,'o');    % landmark 3
legend('sys model');
xlabel('x [meters]');
ylabel('y [meters]');
axis([0 20 0 20]);
grid on;

figure; hold on;
plot(t,phi*180/pi);
legend('sys model');
xlabel('time');
ylabel('\phi [degrees]');